use std::process::Child;
use dark_light::Mode;
use egui::{IconData, Theme, Vec2};
use egui_extras::install_image_loaders;
use egui_flex::{item, Flex};
use backend::config::{Config, Paths, Versions};
use crate::backend::http::HttpState;
use crate::backend::secrets::Account;
use crate::ui::landing::Landing;
use crate::ui::{game_open, Pages};
use crate::ui::launcher::Launcher;

mod ui;
mod backend;

fn main() -> Result<(), eframe::Error> {
    let default_theme: Theme = match dark_light::detect() {
        Ok(Mode::Dark) => Theme::Dark,
        _ => Theme::Light
    };

    let icon = image::load_from_memory(include_bytes!("../assets/Cooking_icon.png")).expect("Failed to load Cooking_icon").to_rgba8();

    let viewport = egui::ViewportBuilder::default()
        .with_title("OSRS Luncher")
        .with_app_id("sex.gaycatgirl.luncher")
        .with_icon(IconData {
            rgba: icon.clone().into_raw(),
            width: icon.width(),
            height: icon.height(),
        })
        .with_fullscreen(true)
        .with_min_inner_size(Vec2 { x: 640.0, y: 480.0 });

    let mut fonts = egui::FontDefinitions::default();
    egui_phosphor::add_to_fonts(&mut fonts, egui_phosphor::Variant::Regular);
    egui_phosphor::add_to_fonts(&mut fonts, egui_phosphor::Variant::Fill);


    let options = eframe::NativeOptions {
        viewport,
        ..Default::default()
    };

    eframe::run_native(
        "osrs luncher",
        options,
        Box::new(|ctx| {
            ctx.egui_ctx.set_fonts(fonts);
            ctx.egui_ctx.set_theme(default_theme);

            install_image_loaders(&ctx.egui_ctx);

            Ok(Box::<App>::default())
        })
    )
}


struct State {
    child: Option<Child>,
    game_open: bool,
    current_page: Pages,
    next_page: Option<Pages>,
}

struct App {
    config: Config,
    account: Option<Account>,
    state: State,
    http_state: HttpState,
    working: bool,
}

impl Default for App {
    fn default() -> Self {
        App {
            config: Config { paths: Paths { runelite_path: "".to_string(), java_path: "".to_string() }, versions: Versions { runelite_version: "".to_string(), runelite_hash: "".to_string() } },
            account: None,
            state: State { child: None, game_open: false, current_page: Pages::Landing(None), next_page: None },
            http_state: HttpState::new(),
            working: false,
        }
    }
}

impl eframe::App for App {
    fn update(&mut self, ctx: &egui::Context, _frame: &mut eframe::Frame) {
        // If we've requested a page change, change to it
        if let Some(next_page) = self.state.next_page.take() {
            self.state.current_page = next_page;

            self.state.next_page = None;
        }

        // Look to see if the child process has exited yet and reset to the launcher if it has
        if let Some(child) = self.state.child.as_mut() {
            if let Ok(Some(_)) = child.try_wait() {
                self.state.game_open = false;
                self.state.child = None;

                self.state.current_page = Pages::Launcher(Launcher::init());
            }
        };

        // Make sure to show the game open page when its open
        if self.state.game_open {
            self.state.current_page = Pages::GameOpen;
        }

        egui::TopBottomPanel::bottom("footer").show(ctx, |ui| {
            Flex::new().w_full().show(ui, |flex| {
                flex.add_ui(item(), |ui| {
                    ui.label(format!("osrs luncher v{}", env!("CARGO_PKG_VERSION")));
                    ui.separator();
                    ui.hyperlink_to("see the source", "https://forge.gaycatgirl.sex/evie/osrs-luncher");
                });

                flex.grow();

                flex.add_ui(item(), |ui| {
                    if self.working {
                        ui.spinner();
                        ui.label("working...");

                        self.working = false;
                    };
                })
            });

        });

        egui::CentralPanel::default().show(ctx, move |ui| {
            ui.heading("osrs luncher");

            ui.separator();

            match &mut self.state.current_page {
                Pages::Landing(landing) => {
                    if let Some(landing) = landing.as_mut() {
                        landing.update(&mut self.state.next_page, &mut self.http_state, &mut self.account, &mut self.config, &mut self.working, ctx, ui)
                    } else {
                        *landing = Some(Landing::init())
                    }
                }
                Pages::Login(login) => {
                    login.update(&mut self.state.next_page, &mut self.account, &mut self.working, ctx, ui)
                }
                Pages::Settings(settings) => {
                    settings.update(&mut self.state.next_page, &mut self.config, ctx, ui)
                }
                Pages::Launcher(launcher) => {
                    launcher.update(&mut self.state.next_page, &mut self.account, &mut self.config, &mut self.state.child, &mut self.state.game_open, ctx, ui);
                }
                Pages::GameOpen => {
                    game_open::update(&mut self.state.child, ctx, ui);
                }
            }
        });
    }
}