use std::fs;
use std::path::Path;
use egui::{Context, ProgressBar, Ui};
use serde_json::{Map, Value};
use crate::backend::config::{Config, CONFIG_DIR};
use crate::backend::http::HttpChannelChunk;
use crate::backend::secrets::Account;
use crate::HttpState;
use crate::ui::launcher::Launcher;
use crate::ui::login::{Login, LoginType};
use crate::ui::Pages;

pub struct Landing {
    stage: LandingStage,
    server_version: Option<String>,
}

#[derive(Clone)]
pub enum LandingStage {
    Config,
    CheckUpdate,
    Verify,
    Download,
    Account,
}

impl Landing {
    pub fn init() -> Self {
        Landing {
            stage: LandingStage::Config,
            server_version: None,
        }
    }

    pub fn update(&mut self,
                  next_page: &mut Option<Pages>,
                  http_state: &mut HttpState,
                  account: &mut Option<Account>,
                  config: &mut Config,
                  working: &mut bool,
                  _ctx: &Context,
                  ui: &mut Ui
    ) {
        match self.stage {
            LandingStage::Config => {
                ui.label("loading config");
                *working = true;

                *config = Config::get_config();

                self.stage = LandingStage::CheckUpdate;
            }
            LandingStage::CheckUpdate => {
                ui.label("checking for updates");
                *working = true;

                // Only do these checks if we're managing the runelite jar
                if Path::new(&config.paths.runelite_path).to_path_buf().parent().unwrap().eq(CONFIG_DIR.get().unwrap()) {
                    if http_state.request_in_progress {
                        if let Some(channel) = &http_state.channel {
                            if let Ok(res) = channel.try_recv() {
                                match res {
                                    HttpChannelChunk::Oneshot(res) => {
                                        let res_json: Map<String, Value>  = serde_json::from_slice(&res.bytes).expect("Invalid json from server");

                                        let server_version = res_json.get("tag_name").expect("Missing tag_name in server response").as_str().unwrap().to_string();

                                        println!("Server version: {}", server_version);

                                        self.server_version = Some(server_version.clone());

                                        if config.versions.runelite_version.ne(&server_version) {
                                            self.stage = LandingStage::Download;
                                        } else {
                                            self.stage = LandingStage::Verify;
                                        }

                                        // We're done, destroy channel and reset fields
                                        http_state.channel = None;
                                        http_state.data = vec![];
                                        http_state.total_size = 0.0;
                                        http_state.current_size = 0.0;
                                        http_state.request_in_progress = false;
                                    }
                                    _ => {
                                        panic!("Invalid!")
                                    }
                                }
                            }
                        }
                    } else {
                        http_state.do_oneshot_request(ehttp::Request::get("https://api.github.com/repos/runelite/launcher/releases/latest"));
                    }
                }
            }
            LandingStage::Verify => {
                ui.label("verifying runelite");
                *working = true;

                self.stage = LandingStage::Account;

                if !Path::new(&config.paths.runelite_path).exists() {
                    self.stage = LandingStage::Download;

                    return;
                }

                // Only do these checks if we're managing the runelite jar
                if Path::new(&config.paths.runelite_path).to_path_buf().parent().unwrap().eq(CONFIG_DIR.get().unwrap()) {
                    if config.versions.runelite_version.is_empty() || config.versions.runelite_hash.is_empty() {
                        self.stage = LandingStage::Download;
                    }

                    let jar = fs::read(&config.paths.runelite_path);
                    match jar {
                        Ok(jar) => {
                            if blake3::hash(&jar).to_hex().to_string() != config.versions.runelite_hash {
                                self.stage = LandingStage::Download;
                            }
                        }
                        _=> {
                            self.stage = LandingStage::Download;
                        }
                    }
                }
            }
            LandingStage::Download => {
                ui.label("downloading runelite");
                *working = true;

                let update_to = self.server_version.clone().expect("Asked to download without version!");
                
                if http_state.request_in_progress {
                    let progress = http_state.current_size / http_state.total_size;

                    ui.add(ProgressBar::new(progress));
                    ui.label(format!("{:.1}%", (progress * 100.0).max(0.0)));

                    if let Some(channel) = &http_state.channel {
                        while let Ok(res) = channel.try_recv(){
                            match res {
                                HttpChannelChunk::Start(total) => {
                                    http_state.total_size = total;
                                }
                                HttpChannelChunk::Data(data) => {
                                    http_state.current_size += data.len() as f32;
                                    http_state.data.extend(data);
                                }
                                HttpChannelChunk::End => {
                                    // We're done, destroy channel, write buffer, and reset fields
                                    http_state.channel = None;

                                    // this isn't the best behaviour (we're writing a good amount of data sync'ed) but its easy and fast so who cares
                                    fs::write(&config.paths.runelite_path, &http_state.data).expect("Failed to write runelite jar");

                                    // same bad behaviour as above
                                    config.versions.runelite_hash = blake3::hash(&http_state.data).to_hex().to_string();
                                    config.versions.runelite_version = update_to;
                                    config.save_config();

                                    http_state.data = vec![];

                                    http_state.total_size = 0.0;
                                    http_state.current_size = 0.0;

                                    http_state.request_in_progress = false;

                                    // TODO: verify or smth

                                    self.stage = LandingStage::Account;
                                    break;
                                }
                                _ => {
                                    panic!("Unexpected http channel chunk");
                                }
                            }
                        }
                    };
                } else {
                    http_state.do_request(ehttp::Request::get(format!("https://github.com/runelite/launcher/releases/download/{}/RuneLite.jar", update_to)));
                }
            }
            LandingStage::Account => {
                ui.label("loading account");
                *working = true;

                *account = Account::get_secrets();

                if account.is_none() {
                    *next_page = Some(Pages::Login(Login::init(LoginType::NoAccount)));
                } else {
                    // TODO: verification that confirms the creds are valid
                    *next_page = Some(Pages::Launcher(Launcher::init()));
                }
            }
        }
    }
}

