use std::cmp::PartialEq;
use egui::{Context, Ui};
use crate::backend::secrets::Account;
use crate::ui::launcher::Launcher;
use crate::ui::Pages;

#[derive(Clone)]
pub struct Login {
    login_type: LoginType,
    login_error_type: Option<LoginErrorType>,
    session_id: String,
    character_name: String,
    character_id: String,
    save_credentials: bool,
}

#[derive(Clone)]
pub enum LoginType {
    NoAccount,
    ExpiredCredentials,
}

#[derive(Clone, PartialEq)]
pub enum LoginErrorType {
    LoggingIn,
    MissingFields,
    FailedToSave
}

impl Login {
    pub fn init(login_type: LoginType) -> Login {
        Login { login_type, login_error_type: None, session_id: "".to_string(), character_name: "".to_string(), character_id: "".to_string(), save_credentials: false }
    }

    pub fn update(&mut self,
                  next_page: &mut Option<Pages>,
                  account: &mut Option<Account>,
                  working: &mut bool,
                  _ctx: &Context,
                  ui: &mut Ui
    ) {
        ui.heading("Login");

        match self.login_type {
            LoginType::NoAccount => {
                ui.label("No account");
            }
            LoginType::ExpiredCredentials => {
                ui.label("Expired credentials");
            }
        }

        ui.horizontal(|ui| {
            ui.label("Character Name: ");

            ui.add(egui::TextEdit::singleline(&mut self.character_name));
        });

        ui.horizontal(|ui| {
            ui.label("Character ID: ");
            ui.add(egui::TextEdit::singleline(&mut self.character_id).password(true));
        });

        ui.horizontal(|ui| {
            ui.label("Session ID: ");
            ui.add(egui::TextEdit::singleline(&mut self.session_id).password(true));
        });

        ui.horizontal(|ui| {
            if ui.button("Login").clicked() {
                self.login_error_type = None;

                if self.character_name.is_empty() | self.character_id.is_empty() | self.session_id.is_empty() {
                    self.login_error_type = Some(LoginErrorType::MissingFields);
                } else {
                    self.login_error_type = Some(LoginErrorType::LoggingIn);

                    let new_account = Account {
                        session_id: self.session_id.clone(),
                        character_id: self.character_id.clone(),
                        character_name: self.character_name.clone(),
                    };

                    *account = Some(new_account.clone());

                    if self.save_credentials && !new_account.insert_secrets() {
                        self.login_error_type = Some(LoginErrorType::FailedToSave);
                    }

                    if self.login_error_type.clone().is_some_and(|x| x.eq(&LoginErrorType::LoggingIn)) {
                        *next_page = Some(Pages::Launcher(Launcher::init()))
                    }
                }
            }

            if let Some(error_type) = self.login_error_type.clone() {
                match error_type {
                    LoginErrorType::MissingFields => {
                        ui.label("Missing fields");
                    }
                    LoginErrorType::LoggingIn => {
                        ui.label("Logging in...");
                        *working = true;
                    }
                    LoginErrorType::FailedToSave => {
                        ui.label("Failed to save credentials.");
                    }
                }
            }
        });

        ui.checkbox(&mut self.save_credentials, "Save credentials?");
    }
}